function [eerror,err_ref] = stoch_diff_referror(domain,xy,P,polyd,x_gal,...
                       error,norv,G_ref,K_ref,xy_ref,x_ref,P_ref,polyd_ref)
%STOCH_DIFF_REFERROR     energy error using stochastic reference solution
%  [eerror, err_ref] = stoch_diff_referror(domain,xy,P,polyd,x_gal, ...
%                     tot_err_est,noarv,G_ref,K_ref,xy_ref,x_ref,P_ref,polyd_ref);
%   input
%          domain        domain identifier  
%          xy            coarse grid coordinate vector
%          P             length of coarse index set
%          polyd         coarse problem polynomial degree  
%          x_gal         coarse grid solution
%          tot_err_est   computed error estimate
%          noarv         number of active random variables | must be fixed
%          G_ref         reference G matrix structure | norv components
%          K_ref         reference K_nbc structure | norv components
%          xy_ref        fine grid coordinate vector
%          x_ref         fine grid solution
%          P_ref         length of the enhanced index set
%          polyd_ref     fine problem polynomial degree 
%
%   output
%          eerror             energy error estimate  
%          err_ref            error vector (dimension of reference solution
%
%   SIFISS function: CEP; 24 July 2015.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester
finest=0;
fprintf('\nstochastic error estimation using reference solution\n')
if domain~=101 & domain~=1, 
   error('Oops.. inappropriate domain specified!'), end

fprintf('number of random variables is %2i\n',norv)

% generate the coarse to fine mapping 
%[~,indset] = stoch_indset(polyd,norv);
%[~,indset_ref] = stoch_indset(polyd_ref,norv);

[indset] = stoch_indset(polyd,norv);
[indset_ref] = stoch_indset(polyd_ref,norv);
indset=sortrows(indset);
indset_ref=sortrows(indset_ref);
indset=double(indset); 
indset_ref=double(indset_ref);

[~,list] = setdiff(indset_ref,indset,'rows');
sameset=setdiff([1:P_ref]', list);
if norm(indset_ref(sameset,:)-indset,inf)~=0 | length(sameset(:,1))~=P, 
    error('Oops.. check the input reference parameters!'), end

% fine grid
  xf=xy_ref(:,1); yf=xy_ref(:,2);
  nfnf=length(xf); nf=sqrt(nfnf);
  fprintf('  fine grid is %3i by %3i | ',nf,nf)
  fprintf('polynomial degree is %2i \n',polyd_ref)
  Xf=reshape(xf,nf,nf); Yf=reshape(yf,nf,nf);
%%%  Xref=reshape(x_ref,nf,nf); 
  Xf=Xf'; Yf=Yf'; 
  ndof=nfnf*P_ref;

% coarse grid
  xc=xy(:,1); yc=xy(:,2); 
  ncnc=length(xc); nc=sqrt(ncnc);
  fprintf('coarse grid is %3i by %3i | ',nc,nc)
  fprintf('polynomial degree is %2i \n',polyd)
  Xc=reshape(xc,nc,nc); Yc=reshape(yc,nc,nc);  
  Xc=Xc'; Yc=Yc';

% initialize X_int
  Xint=zeros(nf,nf,P_ref);

% transform x_gal into (nc)-by-(nc)-by-(P)-tensor
  Xgal=reshape(x_gal,nc,nc,P);

% interpolate the common variables in the two index sets
  for k=1:P
      kref=sameset(k);
      Xgal(:,:,k)=Xgal(:,:,k)';
      Xint(:,:,kref)=interp2(Xc,Yc,Xgal(:,:,k),Xf,Yf,'linear');
      Xint(:,:,kref)=Xint(:,:,kref)';
  end

% transform X_int into column vector
  x_int=reshape(Xint,ndof,1);

% compute the error vector
  err_ref=x_ref-x_int;
  
% debug --- sanity check
fdof=(nfnf+1)/2; cdof=(ncnc+1)/2;
%fdof=(nf+1)/2; cdof=(nc+1)/2;
fprintf('\n\nsanity check: \n')
fprintf('coarse point | x = %9.4e | y = %9.4e\n',xc(cdof), yc(cdof))
fprintf('  fine point | x = %9.4e | y = %9.4e\n',xf(fdof), yf(fdof))
fprintf('  P   x_ref       x_int      difference \n')
for k=1:P_ref
kdof= (k-1)*nfnf+fdof;
fprintf('%3i  %10.4e  %10.4e  %10.4e\n',k,x_ref(kdof),x_int(kdof),err_ref(kdof))
end
fprintf('\n')
% check worst error
if finest,
[maxdiff,kk]=max(abs(err_ref));
zdof=rem(kk,nfnf);
fprintf('  fine point | x = %9.4e | y = %9.4e\n',xf(zdof), yf(zdof))
fprintf('  P   x_ref       x_int      difference \n')
for k=1:P_ref
kdof= (k-1)*nfnf+zdof;
fprintf('%3i  %10.4e  %10.4e  %10.4e\n',k,x_ref(kdof),x_int(kdof),err_ref(kdof))
end
fprintf('\n')
end


% compute the energy error
b_ref = stoch_matvec(err_ref,G_ref,K_ref);
eerror = sqrt(err_ref'*b_ref)/(sqrt(2.0e0)^norv);
fprintf('energy estimate is %8.4e \n',eerror)
fprintf('    effectivity is %8.4e \n',error/eerror)
return
