function [err_gal,err_est_xq,zerr_sq_el] = stoch_diffpost_q1_xqx(xy,ev,bound,q1sol,...
                                 indset,polyd,norv,noarv,KL_DATA,pm_choice,tout)
%STOCH_DIFFPOST_Q1_XQX  computes XQX error estimator for stochastic Q1 solution
% V_{XQ}-estimator: employs the same Q1 basis functions as for the solution
%                   tensorised with the 'detailed set' of polynomials,
%                   uses 'mean-based' bilinear form B_0
% [err_gal,err_est_xq,err_sq_el] = stoch_diffpost_q1_xqx(xy,ev,bound,q1sol,...
%                                 indset,polyd,norv,noarv,KL_DATA,pm_choice,tout);
%   input
%          xy             vertex coordinate vector
%          ev             element mapping matrix
%          bound          boundary vertex vector
%          q1sol          stochastic Q1 solution vector
%          indset         index set of polynomial degrees
%          polyd          maximum polynomial degree in the index set
%          norv           number of random variables
%          noarv          the number of active random variables
%          KL_DATA        data related to KL-expansion
%          pm_choice      p- or M- method for enriching the polynomial space
%          tout           output switch 
%   output
%          err_gal        energy error vector (defined at the nodes)
%          err_est_xq     overall energy error estimate
%          err_sq_el      element error estimate (squared)
%
%   calls functions gausspoints_oned, gausspoints_twod, deriv, qderiv, stoch_gauss_coeff
%   calls functions stoch_intres_q1, stoch_edgeres_q1
%   SIFISS function: DJS;  6 July 2017.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

% check the index set
 [P,norvx] = size(indset); 
 if norvx~=norv, error('error in call to stoch_diffpost_q1_xqx!'), end

  fprintf('computing XQ-error estimator...  \n')

% set new index set depending on pm_choice
  if pm_choice==1
% set new index set by increasing the polynomial degree
     dd=0; pp=1;
     [new_indset] = stoch_indset(polyd+pp,noarv);
     new_indset = [new_indset,zeros(length(new_indset),norv-noarv)];
  elseif pm_choice==2
% set new index set by increasing the number of active random variables
     dd=default('number of additional parameters (default 1)',1);
     pp=0;
     [new_indset] = stoch_indset(polyd,noarv+dd);
     new_indset = [new_indset,zeros(length(new_indset),norv-noarv-dd)];
  elseif pm_choice==3
% set new index set by increasing both the polynomial degree and the number of active random variables
     dd=default('number of additional parameters (default 1)',1);
     pp=1;
     [new_indset] = stoch_indset(polyd+pp,noarv+dd);
     new_indset = [new_indset,zeros(length(new_indset),norv-noarv-dd)];
  end


% find the 'difference' between the new and original index sets
  Q_indset = setdiff(new_indset,indset,'rows');

% find the length of Q_indset
  [Q,xnorv] = size(Q_indset);   % P=p+1 if norv=1

  x=xy(:,1); y=xy(:,2);
  nvtx=length(x);
  nel=length(ev(:,1));

% initialise global stochastic matrices
% a0 = sparse(nvtx*Q,nvtx*Q);
  apq = sparse(nvtx*P,nvtx*Q);
  xl_v=nan(nel,4); yl_v=nan(nel,4);
  err_sq_el=nan(nel,1);


% construct the integration rule
  ngpt=3;
  [oneg,onew] = gausspoints_oned(ngpt);
  [s,t,wt] = gausspoints_twod(oneg,onew);
  nngpt=ngpt^2;

  t1=tic;
% inner loop over elements
  for ivtx = 1:4
      xl_v(:,ivtx) = x(ev(:,ivtx));
      yl_v(:,ivtx) = y(ev(:,ivtx)); 
  end

% initialise local deterministic matrices
  ade = zeros(nel,4,4,norv); adex = zeros(nel,4,4);

% loop over Gauss points
  for igpt = 1:nngpt
      sigpt=s(igpt);
      tigpt=t(igpt);
      wght=wt(igpt);
% evaluate derivatives etc
      [jac_v,invjac_v,phi_v,dphidx_v,dphidy_v] = deriv(sigpt,tigpt,xl_v,yl_v);
      coeff_v = stoch_gauss_coeff(sigpt,tigpt,xl_v,yl_v,norv,KL_DATA);
      m = 0; % mean matrices 
      for j = 1:4
          for i = 1:4
              adex(:,i,j) = adex(:,i,j) + wght * coeff_v(:,m+1) .* dphidx_v(:,i) .* dphidx_v(:,j) .* invjac_v(:);
              adex(:,i,j) = adex(:,i,j) + wght * coeff_v(:,m+1) .* dphidy_v(:,i) .* dphidy_v(:,j) .* invjac_v(:);
          end
      end   
      for m = 1:norv
          for j = 1:4
              for i = 1:4
                  ade(:,i,j,m) = ade(:,i,j,m) + wght * coeff_v(:,m+1) .* dphidx_v(:,i) .* dphidx_v(:,j) .* invjac_v(:);
                  ade(:,i,j,m) = ade(:,i,j,m) + wght * coeff_v(:,m+1) .* dphidy_v(:,i) .* dphidy_v(:,j) .* invjac_v(:);
              end
          end
      end
% end of Gauss point loop
  end

% compute G-matrices
  GQQ = stoch_gmatrices(Q_indset,Q,noarv,norv);
  GPQ = stoch_gmatrices_new(indset,Q_indset);


% perform assembly of global matrices
  m=0;
% initialise global deterministic matrix
  adx = sparse(nvtx,nvtx);
      for krow=1:4
          nrow=ev(:,krow);	 
          for kcol=1:4
              ncol=ev(:,kcol);	  
              adx = adx + sparse(nrow,ncol,adex(:,krow,kcol),nvtx,nvtx);
          end
      end
% tensorise with G-matrices
  apq = apq + kron(GPQ{1}',adx);
      
 % perform assembly of global matrices
  for m=1:norv
% initialise global deterministic matrix
      ad = sparse(nvtx,nvtx);
      for krow=1:4
          nrow=ev(:,krow);	 
          for kcol=1:4
              ncol=ev(:,kcol);	  
              ad = ad + sparse(nrow,ncol,ade(:,krow,kcol,m),nvtx,nvtx);
          end
      end
% tensorise with G-matrices
     apq = apq + kron(GPQ{m+1}',ad);
  end

% compute global residual vector
% (note that its F(v) contribution is zero IF rhs is deterministic
% and, as always assumed, (0,0,...,0) is not in Q_indset)
  f = - q1sol' * apq;
  f=f';

% implement boundary conditions
  matr_dim=length(f);
  nbd=length(bound);
  nint=nvtx-nbd;
  nodes=[1:nvtx]'; 
  intern=setdiff(nodes,bound);
%
  dupl_nodes=1:matr_dim;
  dupl_bound=zeros(nbd*Q,1);
  for k = 1:Q
      l = nvtx*(k-1)*ones(nbd,1);
      dupl_bound((k-1)*nbd+1:k*nbd,1) = l+bound;
  end
  dupl_intern=setdiff(dupl_nodes,dupl_bound);
%
% set boundary conditions
% (note that these are zeros IF b.c. in the problem are deterministic
% and, as always assumed, (0,0,...,0) is not in Q_indset;
% otherwise see the function stoch_nonzerobc.m and
% the function stoch_rhs_multipliers.m needs to be amended!)
  xbd=xy(bound,1); ybd=xy(bound,2);
  bc = zeros(nbd*Q,1);
% sanity check for tensor product structure below
% only the first nbd entries in vector bc can be nonzero
  if Q>1,
  check = find(bc(nbd+1,:)); 
  if ~isempty(check), error('Oops: check BC logic!'), end
  end
  detbc=bc(1:nbd);
%
%  Aerr_gal=a0(dupl_intern,1:matr_dim);
  Adet=adx(intern,1:end);
  res_gal=f(dupl_intern);
  res_gal(1:nint) = res_gal(1:nint) - Adet(:,bound)*detbc;
  Adet=Adet(:,intern);
%
  err_gal=zeros(matr_dim,1);
  err_gal(dupl_bound)=bc;

% forward-backward substitutions ...
  [LA,UA] = lu(Adet);
% loop over the stochastic components
  errde=zeros(size(res_gal)); err_sq=0;
  for sc=1:Q;
  rde=res_gal((sc-1)*nint+1:sc*nint);
  ede = UA\(LA\rde);
  errde((sc-1)*nint+1:sc*nint)= ede;
  %keyboard
  err_det=err_gal((sc-1)*nvtx+1:sc*nvtx);
  err_det(intern)=ede;
  err_sq = err_sq + err_det'*adx*err_det;
  end
  err_gal(dupl_intern) = errde;

% compute energy error estimate
  err_est_xq = sqrt(err_sq);
  

% compute local energy norm of the error estimator
%
% initialisation
  err_sq_el = zeros(nel,1);
%
% transforming err_gal into (nvtx)-by-(Q)-matrix
  err_gal_matr=reshape(err_gal,nvtx,Q);
%
%fprintf('\n debug check  ... this takes time')
%  for el=1:nel
%      for nu=1:Q
%          err_loc = zeros(4,1);
%          for ivtx=1:4
%              err_loc(ivtx,1) = err_gal_matr(ev(el,ivtx),nu);
%          end
%          err_sq_el(el,1) = err_sq_el(el,1) + err_loc'*squeeze(adex(el,1:4,1:4))*err_loc;
%      end
%  end

% vectorized code
 zerr_sq_el = zeros(nel,1);
  zerr_loc = nan(nel,4); 
      for nu=1:Q     
          for ivtx=1:4
              zerr_loc(:,ivtx) = err_gal_matr(ev(:,ivtx),nu);
          end
          for ivtx=1:4
              for jvtx=1:4
              zerr_sq_el(:,1) = zerr_sq_el(:,1) + ...
                                zerr_loc(:,ivtx).*adex(:,ivtx,jvtx).*zerr_loc(:,jvtx);
              end  
          end      
      end
   
% debug check
%   err_diff=err_sq_el-zerr_sq_el;
%  fprintf('\n absolute difference is %7.4e\n',norm(err_diff,inf))

  etime=toc(t1);
  fprintf('  Error estimation took %6.3e seconds\n',etime)
  if tout >1,fprintf('detail space polynomial degree is %g\n',polyd+pp),
     fprintf('detail index set:\n')
     for k=1:Q
         fprintf('%i ',Q_indset(k,1:noarv+dd)); fprintf('\n')
     end
  end
return
