function [err_sq_el,elerr,fe,ade] = stoch_diffpost_q1_yqx(xy,ev,ebound,q1sol,...
                                   eex,hx,hy,indset,polyd,norv,noarv,KL_DATA,pm_choice,tout)
%STOCH_DIFFPOST_Q1_YQX  computes YQX error estimator for stochastic Q1 solution
% V_{YQ}-estimator: employs elementwise Q2-bubbles tensorised with the 
%                   'detailed set' of polynomials, uses 'mean-based' bilinear form B_0
%   [err_sq_el,elerr,fe,ae] = stoch_diffpost_q1_yqx(xy,ev,ebound,q1sol,...
%                             eex,hx,hy,indset,polyd,norv,noarv,KL_DATA,pm_choice,tout);
%   input
%          xy             vertex coordinate vector
%          ev             element mapping matrix
%          ebound         element edge boundary matrix
%          q1sol          stochastic Q1 solution vector
%          eex            element edge connectivity array
%          hx,hy          element mesh sizes
%          indset         index set of polynomial degrees
%          polyd          maximum polynomial degree in the index set
%          norv           number of random variables
%          noarv          the number of active random variables
%          KL_DATA        data related to KL-expansion
%          pm_choice      p- or M- method for enriching the polynomial space
%          tout           output switch
%   output
%          err_sq_el      element error estimate (squared 'energy error')
%          elerr          elementwise error estimate coefficient vector
%          fe             elementwise rhs vectors
%          ae             LDLT factorized element matrices
%
%   calls functions gausspoints_oned, gausspoints_twod, deriv, qderiv, stoch_gauss_coeff
%   calls functions stoch_intres_q1, stoch_edgeres_q1
%   SIFISS function: DJS; 17 June 2017
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

% check the index set
 [P,norvx] = size(indset); 
 if norvx~=norv, error('error in call to stoch_diffpost_q1_yq!'), end

  fprintf('computing YQ-error estimator...  \n')

% set new index set depending on pm_choice
  if pm_choice==1
% set new index set by increasing the polynomial degree
     [new_indset] = stoch_indset(polyd+1,noarv);
     new_indset = [new_indset,zeros(length(new_indset),norv-noarv)];
     if tout >0,fprintf('detail space polynomial degree is %g\n',polyd+1),end
  elseif pm_choice==2
% set new index set by increasing the number of active random variables
     [new_indset] = stoch_indset(polyd,noarv+1);
     new_indset = [new_indset,zeros(length(new_indset),norv-noarv-1)];
     if tout >0,fprintf('detail space is based on %g random variables\n',noarv+1),end
  elseif pm_choice==3
% set new index set by increasing both the polynomial degree and the number of active random variables
     [new_indset] = stoch_indset(polyd+1,noarv+1);
     new_indset = [new_indset,zeros(length(new_indset),norv-noarv-1)];
     if tout >0,fprintf('detail space is based on %g random variables, polynomial degree is %i\n',noarv+1,polyd+1),end
  end


% find the 'difference' between the new and original index sets
  Q_indset = setdiff(new_indset,indset,'rows');

% find the length of Q_indset
  [Q,xnorv] = size(Q_indset);

  x=xy(:,1); y=xy(:,2);
  nel=length(ev(:,1));
  nn=5*Q;
  err_sq_el = zeros(nel,1);
  xl_v=nan(nel,4); yl_v=nan(nel,4);


% construct the integration rule
  ngpt=3;
  [oneg,onew] = gausspoints_oned(ngpt);
  [s,t,wt] = gausspoints_twod(oneg,onew);
  nngpt=ngpt^2;

  t1=tic;
% inner loop over elements
  for ivtx = 1:4
      xl_v(:,ivtx) = x(ev(:,ivtx));
      yl_v(:,ivtx) = y(ev(:,ivtx)); 
  end

% initialise local stochastic and deterministic matrices
%  ae = zeros(nel,nn,nn);
  ade = zeros(nel,5,5);
  elerr=zeros(nn,nel);
  adem = zeros(nel,4,4,4);

  xl_s = zeros(nel,4,4); yl_s = zeros(nel,4,4);
% compute local mid-edge coordinates
  xedge1(:,1)= 0.5*(xl_v(:,1)+xl_v(:,2)); yedge1(:,1)= 0.5*(yl_v(:,1)+yl_v(:,2));
  xedge2(:,1)= 0.5*(xl_v(:,2)+xl_v(:,3)); yedge2(:,1)= 0.5*(yl_v(:,2)+yl_v(:,3));
  xedge3(:,1)= 0.5*(xl_v(:,3)+xl_v(:,4)); yedge3(:,1)= 0.5*(yl_v(:,3)+yl_v(:,4));
  xedge4(:,1)= 0.5*(xl_v(:,4)+xl_v(:,1)); yedge4(:,1)= 0.5*(yl_v(:,4)+yl_v(:,1));
  xcentr(:,1)= 0.25*(xedge1(:)+xedge2(:)+xedge3(:)+xedge4(:));
  ycentr(:,1)= 0.25*(yedge1(:)+yedge2(:)+yedge3(:)+yedge4(:));

% first 
        xl_s(:,1,1) = xl_v(:,1); yl_s(:,1,1) = yl_v(:,1);
        xl_s(:,1,2) = xedge1(:); yl_s(:,1,2) = yedge1(:);
        xl_s(:,1,3) = xcentr(:); yl_s(:,1,3) = ycentr(:);
        xl_s(:,1,4) = xedge4(:); yl_s(:,1,4) = yedge4(:);
% second     
        xl_s(:,2,1) = xedge1(:); yl_s(:,2,1) = yedge1(:);
        xl_s(:,2,2) = xl_v(:,2); yl_s(:,2,2) = yl_v(:,2);
        xl_s(:,2,3) = xedge2(:); yl_s(:,2,3) = yedge2(:);
        xl_s(:,2,4) = xcentr(:); yl_s(:,2,4) = ycentr(:);
% third 
        xl_s(:,3,1) = xcentr(:); yl_s(:,3,1) = ycentr(:);
        xl_s(:,3,2) = xedge2(:); yl_s(:,3,2) = yedge2(:);
        xl_s(:,3,3) = xl_v(:,3); yl_s(:,3,3) = yl_v(:,3);
        xl_s(:,3,4) = xedge3(:); yl_s(:,3,4) = yedge3(:);
% fourth 
        xl_s(:,4,1) = xedge4(:); yl_s(:,4,1) = yedge4(:);
        xl_s(:,4,2) = xcentr(:); yl_s(:,4,2) = ycentr(:);
        xl_s(:,4,3) = xedge3(:); yl_s(:,4,3) = yedge3(:);
        xl_s(:,4,4) = xl_v(:,4); yl_s(:,4,4) = yl_v(:,4);    

% inner-inner loop over subdivided elements
  for subelt=1:4
      for ivtx = 1:4
          xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
          yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
      end
% loop over Gauss points
      for igpt = 1:nngpt
          sigpt=s(igpt);
          tigpt=t(igpt);
          wght=wt(igpt);
% evaluate derivatives etc
          [jac_v,invjac_v,phi_v,dphidx_v,dphidy_v] = deriv(sigpt,tigpt,xl_m,yl_m);
          coeff_v = stoch_gauss_coeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
          for j = 1:4
              for i = 1:4
                  adem(:,subelt,i,j) = adem(:,subelt,i,j)+wght*coeff_v(:,1).*dphidx_v(:,i).*dphidx_v(:,j).*invjac_v(:);
                  adem(:,subelt,i,j) = adem(:,subelt,i,j)+wght*coeff_v(:,1).*dphidy_v(:,i).*dphidy_v(:,j).*invjac_v(:);
              end
          end
% end of Gauss point loop
      end
% end of subdivided element loop
  end

% manual assembly of subelement contributions
% first edge
        ade(:,1,1) = adem(:,1,2,2) + adem(:,2,1,1);
        ade(:,1,2) = adem(:,2,1,3);
        ade(:,1,4) = adem(:,1,2,4); 
        ade(:,1,5) = adem(:,1,2,3) + adem(:,2,1,4);
% second edge
        ade(:,2,2) = adem(:,2,3,3) + adem(:,3,2,2);
        ade(:,2,1) = adem(:,2,3,1);
        ade(:,2,3) = adem(:,3,2,4); 
        ade(:,2,5) = adem(:,2,3,4) + adem(:,3,2,1);
% third edge
        ade(:,3,3) = adem(:,3,4,4) + adem(:,4,3,3);
        ade(:,3,2) = adem(:,3,4,2);
        ade(:,3,4) = adem(:,4,3,1); 
        ade(:,3,5) = adem(:,3,4,1) + adem(:,4,3,2);
% fourth edge
        ade(:,4,4) = adem(:,4,1,1) + adem(:,1,4,4);
        ade(:,4,1) = adem(:,1,4,2);
        ade(:,4,3) = adem(:,4,1,3); 
        ade(:,4,5) = adem(:,4,1,2) + adem(:,1,4,3);
% centroid
        ade(:,5,5) = adem(:,1,3,3) + adem(:,2,4,4)+ adem(:,3,1,1) + adem(:,4,2,2);
        ade(:,5,1) = adem(:,1,3,2) + adem(:,2,4,1);
        ade(:,5,2) = adem(:,2,4,3) + adem(:,3,1,2);
        ade(:,5,3) = adem(:,3,1,4) + adem(:,4,2,3);
        ade(:,5,4) = adem(:,4,2,1) + adem(:,1,3,4); 

% compute G-matrices
  G = stoch_gmatrices(Q_indset,Q,noarv,norv);

% assemble local stiffness matrices by tensorising with G-matrices
%  [nzi,nzj,g] = find(G{1});
%  for t=1:length(nzi)
%      i=nzi(t); j=nzj(t);
%      ae(:,(5*(i-1)+1):(5*i),(5*(j-1)+1):(5*j)) = ae(:,(5*(i-1)+1):(5*i),(5*(j-1)+1):(5*j)) + g(t)*ade(:,1:5,1:5);
%  end

% compute local rhs-vectors
  res_int = stoch_intres_q1_yq(xy,xl_s,yl_s,ev,q1sol,indset,P,Q_indset,norv,noarv,KL_DATA);
  res_edge = stoch_edgeres_q1_yqx(xy,ev,ebound,q1sol,eex,hx,hy,indset,P,Q_indset,noarv,KL_DATA);
  fe = res_int - res_edge;
   % debug
        if tout>1,
            elt=99;
        fprintf('right-hand side vector \n') 
        disp(squeeze(fe(elt,1:end)))  
        end

% solve for local estimate (sequential code)
%  for ielem = 1:nel
%      elerr(:,ielem) = squeeze(ae(ielem,1:nn,1:nn))\(fe(ielem,1:nn)');
%  end
%
% solve for local estimate (vectorised code)
% LDLT factorization
% redefine nn
  nn=5;
  dd=zeros(nel,nn); rr=zeros(nel,nn);
  for kk=1:nn-1
      for pp=1:kk-1;
          rr(1:nel,pp)=dd(1:nel,pp).*ade(1:nel,kk,pp);
      end
      dd(1:nel,kk)=ade(1:nel,kk,kk);
      for pp=1:kk-1;
          dd(1:nel,kk)= dd(1:nel,kk)- ade(1:nel,kk,pp).*rr(1:nel,pp);
      end
      for ii=kk+1:nn
          for pp=1:kk-1;
              ade(1:nel,ii,kk)=ade(1:nel,ii,kk)-ade(1:nel,ii,pp).*rr(1:nel,pp);
          end
          ade(1:nel,ii,kk)=ade(1:nel,ii,kk)./dd(1:nel,kk);
      end
  end
  for pp=1:nn-1;
      rr(1:nel,pp)=dd(1:nel,pp).*ade(1:nel,nn,pp);
  end
  dd(1:nel,nn)=ade(1:nel,nn,nn);
  for pp=1:nn-1;
      dd(1:nel,nn)= dd(1:nel,nn)- ade(1:nel,nn,pp).*rr(1:nel,pp);
  end
% overwrite diagonal entries
  for kk=1:nn
      ade(1:nel,kk,kk)= dd(1:nel,kk);
  end
% forward-backward substitutions ...
% loop over the stochastic components
  elerr=zeros(nn*Q,nel); 
  for nnp=1:Q;
  fde=fe(:,(nnp-1)*5+1:nnp*5);
    if tout>1, fprintf('%g right-hand side\n',nnp),
    disp(squeeze(fde(elt,1:end))),end
  xdx = element_lusolve(ade,fde);
  elerr((nnp-1)*5+1:nnp*5,1:nel)=xdx';
  end
  xx=elerr';
%
  for svtx=1:nn*Q
      err_sq_el(:) = err_sq_el(:) + fe(:,svtx) .* elerr(svtx,:)';
  end

  etime=toc(t1);
  fprintf('  Error estimation took %6.3e seconds\n',etime)

  return

