function [edgeres] = stoch_edgeres_q1_ypx(xy,ev,ebound,q1sol,eex,hx,hy,indset,P,norv,noarv,KL_DATA)
%STOCH_EDGERES_Q1_YPX  computes YPX edge residuals for stochastic Q1 solution
%   [edgeres] = stoch_edgeres_q1_ypx(xy,ev,ebound,q1sol,eex,hx,hy,indset,P,norv,noarv,KL_DATA);
%   input
%          xy           vertex coordinate vector
%          ev           element mapping matrix
%          ebound       element edge boundary matrix
%          q1sol        stochastic Q1 solution vector
%          eex          element edge connectivity array
%          hx,hy        element mesh sizes
%          indset       index set of polynomial degrees
%          P            length of the index set
%          norv         number of random variables
%          noarv        number of active random variables
%          KL_DATA      data related to KL-expansion
%   output
%          edgeres      edge residuals
%
%   calls functions q1fluxjmps, gausspoints_oned, stoch_gauss_coeff, qderiv
%   SIFISS function: DJS; 5 July 2017
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

  x=xy(:,1); y=xy(:,2); nvtx=length(x);
  nel=length(ev(:,1));
  
% construct the integration rule
  ngpt=3;
  [oneg,onew] = gausspoints_oned(ngpt);

% initialisation
  edgeres = zeros(nel,5*P);
  temp_res = zeros(nel,4,P,noarv+1);
  xl_v=nan(nel,4); yl_v=nan(nel,4);

  % inner loop over elements
  for ivtx = 1:4
      xl_v(:,ivtx) = x(ev(:,ivtx));
      yl_v(:,ivtx) = y(ev(:,ivtx)); 
  end

% loop over Gauss points
  for igpt=1:ngpt
      sigpt=oneg(igpt);
      sigpt_l=(sigpt-1.0e0)/2; sigpt_r=(sigpt+1.0e0)/2;
      wigpt=onew(igpt);
% save coefficient data
       coeff_l_1 = stoch_gauss_coeff(sigpt_l,-1,xl_v,yl_v,noarv,KL_DATA);
       coeff_r_1 = stoch_gauss_coeff(sigpt_r,-1,xl_v,yl_v,noarv,KL_DATA);
       [jac_v,invjac_v,phi_v_1,dphidx_v,dphidy_v] = deriv(sigpt,-1,xl_v,yl_v);
%
       coeff_l_2 = stoch_gauss_coeff(1,sigpt_l,xl_v,yl_v,noarv,KL_DATA);
       coeff_r_2 = stoch_gauss_coeff(1,sigpt_r,xl_v,yl_v,noarv,KL_DATA);
       [jac_v,invjac_v,phi_v_2,dphidx_v,dphidy_v] = deriv(1,sigpt,xl_v,yl_v);
%
       coeff_l_3 = stoch_gauss_coeff(sigpt_l,1,xl_v,yl_v,noarv,KL_DATA);
       coeff_r_3 = stoch_gauss_coeff(sigpt_r,1,xl_v,yl_v,noarv,KL_DATA);
       [jac_v,invjac_v,phi_v_3,dphidx_v,dphidy_v] = deriv(sigpt,1,xl_v,yl_v);
%
       coeff_l_4 = stoch_gauss_coeff(-1,sigpt_l,xl_v,yl_v,noarv,KL_DATA);
       coeff_r_4 = stoch_gauss_coeff(-1,sigpt_r,xl_v,yl_v,noarv,KL_DATA);
       [jac_v,invjac_v,phi_v_4,dphidx_v,dphidy_v] = deriv(-1,sigpt,xl_v,yl_v); 
% loop over indices in the index set
      for k=1:P
          njmp_l = q1fluxjmps(q1sol((k-1)*nvtx+1:k*nvtx,1),eex,xy,ev,ebound,sigpt_l);
          njmp_r = q1fluxjmps(q1sol((k-1)*nvtx+1:k*nvtx,1),eex,xy,ev,ebound,sigpt_r);
% loop over stochastic variables
          for m = 0:noarv
              temp_res(:,1,k,m+1) = temp_res(:,1,k,m+1)+(1/2)*wigpt*coeff_l_1(:,m+1).*njmp_l(:,1).*phi_v_1(:,2).*hx(:)*(1/4);
              temp_res(:,1,k,m+1) = temp_res(:,1,k,m+1)+(1/2)*wigpt*coeff_r_1(:,m+1).*njmp_r(:,1).*phi_v_1(:,1).*hx(:)*(1/4);
%
              temp_res(:,2,k,m+1) = temp_res(:,2,k,m+1)+(1/2)*wigpt*coeff_l_2(:,m+1).*njmp_l(:,2).*phi_v_2(:,3).*hy(:)*(1/4);
              temp_res(:,2,k,m+1) = temp_res(:,2,k,m+1)+(1/2)*wigpt*coeff_r_2(:,m+1).*njmp_r(:,2).*phi_v_2(:,2).*hy(:)*(1/4);
%
              temp_res(:,3,k,m+1) = temp_res(:,3,k,m+1)+(1/2)*wigpt*coeff_l_3(:,m+1).*njmp_l(:,3).*phi_v_3(:,3).*hx(:)*(1/4);
              temp_res(:,3,k,m+1) = temp_res(:,3,k,m+1)+(1/2)*wigpt*coeff_r_3(:,m+1).*njmp_r(:,3).*phi_v_3(:,4).*hx(:)*(1/4);
%
              temp_res(:,4,k,m+1) = temp_res(:,4,k,m+1)+(1/2)*wigpt*coeff_l_4(:,m+1).*njmp_l(:,4).*phi_v_4(:,4).*hy(:)*(1/4);
              temp_res(:,4,k,m+1) = temp_res(:,4,k,m+1)+(1/2)*wigpt*coeff_r_4(:,m+1).*njmp_r(:,4).*phi_v_4(:,1).*hy(:)*(1/4);
          end   % end of the loop over stochastic variables
      end       % end of the loop over indices in the index set
  end           % end of Gauss points loop

% compute G-matrices
  G = stoch_gmatrices(indset,P,noarv,norv);

% building edge residual vectors
  for m = 0:noarv
      [nzi,nzj,g] = find(G{m+1});
      for t=1:length(nzi)
          i=nzi(t); j=nzj(t);
          edgeres(:,(5*(j-1)+1):(5*j-1)) = edgeres(:,(5*(j-1)+1):(5*j-1)) + g(t)*temp_res(:,1:4,i,m+1);
      end
  end

  return

