function [edgeres] = stoch_edgeres_q2_ypx(xy,xl_s,yl_s,mv,mbound,q2sol,eex,hx,hy,indset,P,norv,noarv,KL_DATA)
%STOCH_EDGERES_Q2_YPX  computes YPX edge residuals for stochastic Q2 solution
%   [edgeres] = stoch_edgeres_q2_ypx(xy,mv,mbound,q2sol,eex,hx,hy,indset,P,norv,noarv,KL_DATA);
%   input
%          xy           vertex coordinate vector
%          mv           element mapping matrix
%          mbound       element edge boundary matrix
%          q2sol        stochastic Q1 solution vector
%          eex          element edge connectivity array
%          hx,hy        element mesh sizes
%          indset       index set of polynomial degrees
%          P            length of the index set
%          norv         number of random variables
%          noarv        number of active random variables
%          KL_DATA      data related to KL-expansion
%   output
%          edgeres      edge residuals
%
%   calls functions q2fluxjmps, gausspoints_oned, stoch_gauss_coeff, qqderiv
%   SIFISS function:  DJS 5 July 2017
% Copyright (c) 2017 A. Crowder, D. Silvester

neumannb=[];
error_node=[10;11;12;13;14;15;16;17;18;20;22;24];

N_error_node=length(error_node); nn=N_error_node;

x=xy(:,1); y=xy(:,2); nvtx=length(x);
mel=length(mv(:,1));

% construct the integration rule
ngpt=3; ng=ngpt;
[oneg,onew] = gausspoints_oned(ngpt);

% initialisation
edgeres = zeros(mel,nn*P);
temp_res = zeros(mel,nn,P,noarv+1);
xl_v=nan(mel,4); yl_v=nan(mel,4);

% inner loop over elements
for ivtx = 1:4
    xl_v(:,ivtx) = x(mv(:,ivtx));
    yl_v(:,ivtx) = y(mv(:,ivtx));
end

% subelement vertices
for ivtx = 1:4
    xl_m_1(:,ivtx) = xl_s(:,1,ivtx);
    yl_m_1(:,ivtx) = yl_s(:,1,ivtx);
end
for ivtx = 1:4
    xl_m_2(:,ivtx) = xl_s(:,2,ivtx);
    yl_m_2(:,ivtx) = yl_s(:,2,ivtx);
end
for ivtx = 1:4
    xl_m_3(:,ivtx) = xl_s(:,3,ivtx);
    yl_m_3(:,ivtx) = yl_s(:,3,ivtx);
end
for ivtx = 1:4
    xl_m_4(:,ivtx) = xl_s(:,4,ivtx);
    yl_m_4(:,ivtx) = yl_s(:,4,ivtx);
end

% loop over Gauss points
for igpt=1:ngpt
    sigpt=oneg(igpt);
    
    sigpt_l=(sigpt-1.0e0)/2; sigpt_r=(sigpt+1.0e0)/2;
    
    wigpt=onew(igpt);
    
    coeff_l_1 = stoch_gauss_coeff(sigpt_l,-1,xl_v,yl_v,noarv,KL_DATA);
    coeff_r_1 = stoch_gauss_coeff(sigpt_r,-1,xl_v,yl_v,noarv,KL_DATA);
    [phi_v_l_1,~,~] = qderiv(sigpt,-1,xl_m_1,yl_m_1);
    [phi_v_r_1,~,~] = qderiv(sigpt,-1,xl_m_2,yl_m_2);
    
    coeff_l_2 = stoch_gauss_coeff(1,sigpt_l,xl_v,yl_v,noarv,KL_DATA);
    coeff_r_2 = stoch_gauss_coeff(1,sigpt_r,xl_v,yl_v,noarv,KL_DATA);
    [phi_v_l_2,~,~] = qderiv(1,sigpt,xl_m_2,yl_m_2);
    [phi_v_r_2,~,~] = qderiv(1,sigpt,xl_m_3,yl_m_3);
    
    coeff_l_3 = stoch_gauss_coeff(sigpt_l,1,xl_v,yl_v,noarv,KL_DATA);
    coeff_r_3 = stoch_gauss_coeff(sigpt_r,1,xl_v,yl_v,noarv,KL_DATA);
    [phi_v_l_3,~,~] = qderiv(sigpt,1,xl_m_3,yl_m_3);
    [phi_v_r_3,~,~] = qderiv(sigpt,1,xl_m_4,yl_m_4);
    
    coeff_l_4 = stoch_gauss_coeff(-1,sigpt_l,xl_v,yl_v,noarv,KL_DATA);
    coeff_r_4 = stoch_gauss_coeff(-1,sigpt_r,xl_v,yl_v,noarv,KL_DATA);
    [phi_v_l_4,~,~] = qderiv(-1,sigpt,xl_m_4,yl_m_4);
    [phi_v_r_4,~,~] = qderiv(-1,sigpt,xl_m_1,yl_m_1);
    
    %  loop over indices in the index set
    for k=1:P
        
        njmp_l = q2fluxjmps(q2sol((k-1)*nvtx+1:k*nvtx,1),eex,xy,mv,mbound,neumannb,sigpt_l);
        njmp_r = q2fluxjmps(q2sol((k-1)*nvtx+1:k*nvtx,1),eex,xy,mv,mbound,neumannb,sigpt_r);
               
        for m = 0:noarv

            temp_res(:,1,k,m+1) = temp_res(:,1,k,m+1)+(1/2)*wigpt*coeff_l_1(:,m+1).*njmp_l(:,1).*phi_v_l_1(:,5).*hx(:)*(1/4);
            temp_res(:,2,k,m+1) = temp_res(:,2,k,m+1)+(1/2)*wigpt*coeff_r_1(:,m+1).*njmp_r(:,1).*phi_v_r_1(:,5).*hx(:)*(1/4);
            
            temp_res(:,3,k,m+1) = temp_res(:,3,k,m+1)+(1/2)*wigpt*coeff_l_2(:,m+1).*njmp_l(:,2).*phi_v_l_2(:,6).*hy(:)*(1/4);
            temp_res(:,4,k,m+1) = temp_res(:,4,k,m+1)+(1/2)*wigpt*coeff_r_2(:,m+1).*njmp_r(:,2).*phi_v_r_2(:,6).*hy(:)*(1/4);
            
            temp_res(:,6,k,m+1) = temp_res(:,6,k,m+1)+(1/2)*wigpt*coeff_l_3(:,m+1).*njmp_l(:,3).*phi_v_l_3(:,7).*hx(:)*(1/4);
            temp_res(:,5,k,m+1) = temp_res(:,5,k,m+1)+(1/2)*wigpt*coeff_r_3(:,m+1).*njmp_r(:,3).*phi_v_r_3(:,7).*hx(:)*(1/4);
            
            temp_res(:,8,k,m+1) = temp_res(:,8,k,m+1)+(1/2)*wigpt*coeff_l_4(:,m+1).*njmp_l(:,4).*phi_v_l_4(:,8).*hy(:)*(1/4);
            temp_res(:,7,k,m+1) = temp_res(:,7,k,m+1)+(1/2)*wigpt*coeff_r_4(:,m+1).*njmp_r(:,4).*phi_v_r_4(:,8).*hy(:)*(1/4);
            
        end
        
    end       % end of the loop over indices in the index set
end          % end of Gauss points loop

% compute G-matrices
G = stoch_gmatrices(indset,P,noarv,norv);

% building edge residual vectors
for m = 0:noarv
    [nzi,nzj,g] = find(G{m+1});
    for t=1:length(nzi)
        i=nzi(t); j=nzj(t);
        edgeres(:,(nn*(j-1)+1):nn*j) = edgeres(:,(nn*(j-1)+1):nn*j) + g(t)*temp_res(:,1:nn,i,m+1);
    end
end

return

