function [a,f] = stoch_femq1_diff(xy,ev,indset,P,norv,noarv,KL_DATA)
%STOCH_FEMQ1_DIFF stochastic coefficient matrix generation using kron
%   [A,f] = femq1_diff(xy,ev,indset,P,norv,noarv,KL_DATA);
%   input
%          xy         vertex coordinate vector  
%          ev         element mapping matrix
%          indset     index set of polynomial degrees
%          P          length of the index set
%          norv       number of random variables
%          noarv      the number of active random variables
%          KL_DATA    data related to KL-expansion
%   output
%          A          stiffness matrix
%          f          rhs vector
%
%   Natural boundary conditions apply. Dirichlet conditions
%   must be explicitly enforced by calling function stoch_nonzerobc.
%   SIFISS function: DJS; 5 July 2017.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester


  x=xy(:,1); y=xy(:,2);
  nvtx=length(x);
  nel=length(ev(:,1));
%%%%%%  lx=max(x)-min(x); ly=max(y)-min(y);   THESE ARE NOT USED?
  hx=max(diff(x)); hy=max(diff(y));

  fprintf('setting up stochastic Q1 diffusion matrices...  ')

% initialise global stochastic matrix and rhs vector
  a = sparse(nvtx*P,nvtx*P);
  f = zeros(nvtx*P,1);

% construct the integration rule
  ngpt=7;
  [oneg,onew] = gausspoints_oned(ngpt);
  [s,t,wt] = gausspoints_twod(oneg,onew);
  nngpt=ngpt^2;

% inner loop over elements    
  for ivtx = 1:4
      xl_v(:,ivtx) = x(ev(:,ivtx));
      yl_v(:,ivtx) = y(ev(:,ivtx)); 
  end
  ade = zeros(nel,4,4,noarv+1);
  fde = zeros(nel,4,noarv+1);

% loop over Gauss points
  for igpt = 1:nngpt
      sigpt=s(igpt);
      tigpt=t(igpt);
      wght=wt(igpt);
% evaluate derivatives etc
      [jac,invjac,phi,dphidx,dphidy] = deriv(sigpt,tigpt,xl_v,yl_v);
      rhs = stoch_gauss_source(sigpt,tigpt,xl_v,yl_v,noarv);
      coeff = stoch_gauss_coeff(sigpt,tigpt,xl_v,yl_v,noarv,KL_DATA);
      for m = 0:noarv
          for j = 1:4
              for i = 1:4
                  ade(:,i,j,m+1) = ade(:,i,j,m+1) + wght * coeff(:,m+1) .* dphidx(:,i) .* dphidx(:,j) .* invjac(:);
                  ade(:,i,j,m+1) = ade(:,i,j,m+1) + wght * coeff(:,m+1) .* dphidy(:,i) .* dphidy(:,j) .* invjac(:);
              end
              fde(:,j,m+1) = fde(:,j,m+1)  + wght * rhs(:,m+1) .* phi(:,j) .* jac(:);
          end
      end
% end of Gauss point loop
  end

% compute G-matrices and rhs-multipliers
  G = stoch_gmatrices(indset,P,noarv,noarv);
  [rhs_ind,beta] = stoch_rhs_multipliers(indset,P,norv);

% perform assembly of global matrices and source vector
  for m=0:noarv
% initialise global deterministic matrix and rhs vector
      ad = sparse(nvtx,nvtx);
      fd = zeros(nvtx,1);
      for krow=1:4
          nrow=ev(:,krow);	 
          for kcol=1:4
              ncol=ev(:,kcol);	  
              ad = ad + sparse(nrow,ncol,ade(:,krow,kcol,m+1),nvtx,nvtx);
          end
          fd(nrow,1) = fd(nrow,1) + fde(:,krow,m+1);
      end
% tensorise with G-matrices
      a = a + kron(G{m+1},ad);
% assemble the rhs vector
      ind_m = rhs_ind(1,m+1);
      if ind_m>0
         f((nvtx*(ind_m-1)+1):(nvtx*ind_m),1) = fd(:,1)*beta(1,m+1)*(sqrt(2))^noarv;
      end
  end

fprintf('done\n')
return

