function [K,f] = stoch_femq2_setup(xy,mv,indset,P,norv,noarv,KL_DATA)
%STOCH_FEMQ2_SETUP assembled biquadratic stochastic coefficient matrix generator
%   [K,f] = stoch_femq2_setup(xy,mv,indset,P,norv,noarv,KL_DATA);
%   input
%          xy         vertex coordinate vector  
%          mv         element mapping matrix
%          indset     index set of polynomial degrees
%          P          length of the index set
%          norv       number of random variables
%          noarv      the number of active random variables
%          KL_DATA    data related to KL-expansion
%   output
%          K          cell structure with noarv (nKi $\times$ nKi) matrices
%          f          rhs vector
%
%   Natural boundary conditions apply. Dirichlet conditions
%   must be explicitly enforced by calling function stoch_imposebc.
%   SIFISS function: DJS; 5 July 2017.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

  x=xy(:,1); y=xy(:,2);
  nvtx=length(x);
  nel=length(mv(:,1));
  hx=max(diff(x)); hy=max(diff(y));

  fprintf('setting up stochastic Q2 diffusion matrices...  ')

% initialise global stochastic matrix and rhs vector
  a = sparse(nvtx*P,nvtx*P);
  f = zeros(nvtx*P,1);

% construct the integration rule
  ngpt=7;
  [oneg,onew] = gausspoints_oned(ngpt);
  [s,t,wt] = gausspoints_twod(oneg,onew);
  nngpt=ngpt^2;

% inner loop over elements
  for ivtx = 1:4
      xl_v(:,ivtx) = x(mv(:,ivtx));
      yl_v(:,ivtx) = y(mv(:,ivtx)); 
  end
  ade = zeros(nel,9,9,noarv+1);
  fde = zeros(nel,9,noarv+1);

% loop over Gauss points
  for igpt = 1:nngpt
      sigpt=s(igpt);
      tigpt=t(igpt);
      wght=wt(igpt);
% evaluate derivatives etc
      [jac,invjac,phi,dphidx,dphidy] = deriv(sigpt,tigpt,xl_v,yl_v);
      rhs = stoch_gauss_source(sigpt,tigpt,xl_v,yl_v,noarv);
      coeff = stoch_gauss_coeff(sigpt,tigpt,xl_v,yl_v,noarv,KL_DATA);
      [psi,dpsidx,dpsidy] = qderiv(sigpt,tigpt,xl_v,yl_v);
      for m = 0:noarv;
          for j = 1:9
              for i = 1:9
                  ade(:,i,j,m+1) = ade(:,i,j,m+1) + wght * coeff(:,m+1) .* dpsidx(:,i) .* dpsidx(:,j) .* invjac(:);
                  ade(:,i,j,m+1) = ade(:,i,j,m+1) + wght * coeff(:,m+1) .* dpsidy(:,i) .* dpsidy(:,j) .* invjac(:);
              end
              fde(:,j,m+1) = fde(:,j,m+1) + wght * rhs(:,m+1) .* psi(:,j) .* jac(:);
          end
      end
% end of Gauss point loop
  end

% construct structure to hold matrices K_1,...,K_{noarv}
 K = cell(1,noarv+1);
  
% compute rhs-multipliers
  [rhs_ind,beta] = stoch_rhs_multipliers(indset,P,norv);

% perform assembly of global matrices and source vector
  for m=0:noarv;
% initialise global deterministic matrix and rhs vector
      ad = sparse(nvtx,nvtx);
      fd = zeros(nvtx,1);
      for krow=1:9
          nrow=mv(:,krow);	 
          for kcol=1:9
              ncol=mv(:,kcol);	  
              ad = ad + sparse(nrow,ncol,ade(:,krow,kcol,m+1),nvtx,nvtx);
          end
          fd(nrow,1) = fd(nrow,1) + fde(:,krow,m+1);
      end
      
% save assembled matrix in the cell structure
   K{m+1} = ad;

% assemble the rhs vector
      ind_m = rhs_ind(1,m+1);
      if ind_m>0
f((nvtx*(ind_m-1)+1):(nvtx*ind_m),1) = fd(:,1)*beta(1,m+1);
      end
  end

fprintf('done\n')
return

