function [intres] = stoch_intres_q2_yp(xy,xl_s,yl_s,mv,q2sol,indset,P,norv,noarv,KL_DATA)
%STOCH_INTRES_Q2_YP computes YP elementwise interior residuals for stochastic Q2 solution
%   [intres] = stoch_intres_q2_yp(xy,mv,q2sol,indset,P,norv,noarv,KL_DATA);
%   input
%          xy         vertex coordinate vector
%          mv         element mapping matrix
%          q2sol      stochastic Q2 solution vector
%          indset     index set of polynomial degrees
%          P          length of the index set
%          norv       number of random variables
%          noarv      the number of active random variables
%          KL_DATA    data related to KL-expansion
%   output
%          intres     interior residuals
%
%   calls functions gausspoints_oned, gausspoints_twod, deriv, qderiv, qqderiv
%   calls stoch_gauss_source, stoch_gauss_coeff, stoch_gauss_gradcoeff, stoch_rhs_multipliers
%   SIFISS function:  DJS 5 July 2017
% Copyright (c) 2017 A. Crowder, T. Gergelits, D. Silvester

error_node=[10;11;12;13;14;15;16;17;18;20;22;24];

N_error_node=length(error_node); nn=N_error_node;

x=xy(:,1); y=xy(:,2); nvtx=length(x);
mel=length(mv(:,1));

% construct the integration rule
ngpt=3; 
[oneg,onew] = gausspoints_oned(ngpt);
[s,t,wt] = gausspoints_twod(oneg,onew);
nngpt=ngpt^2; ng=ngpt;

% inner loop over elements
for ivtx = 1:4
    xl_v(:,ivtx) = x(mv(:,ivtx));
    yl_v(:,ivtx) = y(mv(:,ivtx));
end
for ivtx=1:9
    for k=1:P
        sl_v(:,ivtx+9*(k-1)) = q2sol(nvtx*(k-1)+mv(:,ivtx));
    end
end

% initialisation
intres = zeros(mel,nn*P);
fde = zeros(mel,nn,noarv+1);
bde = zeros(mel,nn,9,noarv+1);

bdem = zeros(mel,4,5,9,noarv+1);
fdem = zeros(mel,4,5,noarv+1);

% 1st sub-element: [-1,0]x[-1,0]
subelt=1;
for ivtx = 1:4
    xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
    yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
end
% loop over Gauss points
for igpt = 1:nngpt
    sigpt=s(igpt);
    tigpt=t(igpt);
    sigptloc=(sigpt-1.0e0)/2;
    tigptloc=(tigpt-1.0e0)/2;
    wght=wt(igpt);
    
    % evaluate derivatives etc
    
    [jac_m,~,~,~,~] = deriv(sigpt,tigpt,xl_m,yl_m);
    [~,invjac_v,~,~,~] = deriv(sigptloc,tigptloc,xl_v,yl_v);    
    
    rhs = stoch_gauss_source(sigpt,tigpt,xl_m,yl_m,noarv);
    coeff = stoch_gauss_coeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
    [dcoeffdx,dcoeffdy] = stoch_gauss_gradcoeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
    [phi,~,~] = qderiv(sigpt,tigpt,xl_m,yl_m);
    [~,dpsidx,dpsidy] = qderiv(sigptloc,tigptloc,xl_v,yl_v);
    [dpsi2dx2,dpsi2dy2] = qderiv_2(sigptloc,tigptloc,xl_v,yl_v);
    deltapsi = dpsi2dx2+dpsi2dy2;
    
    for m = 0:noarv
        for j = 1:5  % sub-element vertices
            
            fdem(:,subelt,j,m+1) = fdem(:,subelt,j,m+1) + wght*rhs(:,m+1) .* phi(:,j+4) .* jac_m(:);
            
            for i = 1:9  % element vertices
                
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*dcoeffdx(:,m+1).*dpsidx(:,i).*phi(:,j+4).*invjac_v(:).*jac_m(:);
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*dcoeffdy(:,m+1).*dpsidy(:,i).*phi(:,j+4).*invjac_v(:).*jac_m(:);
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*coeff(:,m+1).*deltapsi(:,i).*phi(:,j+4).*(invjac_v(:).^2).*jac_m(:);
                
            end
        end
    end
    % end of Gauss point loop
end

% 2nd sub-element: [0,1]x[-1,0]
subelt=2;
for ivtx = 1:4
    xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
    yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
end
% loop over Gauss points
for igpt = 1:nngpt
    sigpt=s(igpt);
    tigpt=t(igpt);
    sigptloc=(sigpt+1.0e0)/2;
    tigptloc=(tigpt-1.0e0)/2;
    wght=wt(igpt);
    
    % evaluate derivatives etc
    
    [jac_m,~,~,~,~] = deriv(sigpt,tigpt,xl_m,yl_m);
    [~,invjac_v,~,~,~] = deriv(sigptloc,tigptloc,xl_v,yl_v);    
    
    rhs = stoch_gauss_source(sigpt,tigpt,xl_m,yl_m,noarv);
    coeff = stoch_gauss_coeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
    [dcoeffdx,dcoeffdy] = stoch_gauss_gradcoeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
    [phi,~,~] = qderiv(sigpt,tigpt,xl_m,yl_m);
    [~,dpsidx,dpsidy] = qderiv(sigptloc,tigptloc,xl_v,yl_v);
    [dpsi2dx2,dpsi2dy2] = qderiv_2(sigptloc,tigptloc,xl_v,yl_v);
    deltapsi = dpsi2dx2+dpsi2dy2;
    
    for m = 0:noarv
        for j = 1:5  % sub-element vertices
            
            fdem(:,subelt,j,m+1) = fdem(:,subelt,j,m+1) + wght*rhs(:,m+1) .* phi(:,j+4) .* jac_m(:);
            
            for i = 1:9  % element vertices
                
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*dcoeffdx(:,m+1).*dpsidx(:,i).*phi(:,j+4).*invjac_v(:).*jac_m(:);
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*dcoeffdy(:,m+1).*dpsidy(:,i).*phi(:,j+4).*invjac_v(:).*jac_m(:);
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*coeff(:,m+1).*deltapsi(:,i).*phi(:,j+4).*(invjac_v(:).^2).*jac_m(:);
                
            end
        end
    end
    % end of Gauss point loop
end

% 3rd sub-element: [0,1]x[0,1]
subelt=3;
for ivtx = 1:4
    xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
    yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
end
% loop over Gauss points
for igpt = 1:nngpt
    sigpt=s(igpt);
    tigpt=t(igpt);
    sigptloc=(sigpt+1.0e0)/2;
    tigptloc=(tigpt+1.0e0)/2;
    wght=wt(igpt);
    
    % evaluate derivatives etc
    
    [jac_m,~,~,~,~] = deriv(sigpt,tigpt,xl_m,yl_m);
    [~,invjac_v,~,~,~] = deriv(sigptloc,tigptloc,xl_v,yl_v);    
    
    rhs = stoch_gauss_source(sigpt,tigpt,xl_m,yl_m,noarv);
    coeff = stoch_gauss_coeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
    [dcoeffdx,dcoeffdy] = stoch_gauss_gradcoeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
    [phi,~,~] = qderiv(sigpt,tigpt,xl_m,yl_m);
    [~,dpsidx,dpsidy] = qderiv(sigptloc,tigptloc,xl_v,yl_v);
    [dpsi2dx2,dpsi2dy2] = qderiv_2(sigptloc,tigptloc,xl_v,yl_v);
    deltapsi = dpsi2dx2+dpsi2dy2;
    
    for m = 0:noarv
        for j = 1:5  % sub-element vertices
            
            fdem(:,subelt,j,m+1) = fdem(:,subelt,j,m+1) + wght*rhs(:,m+1) .* phi(:,j+4) .* jac_m(:);
            
            for i = 1:9  % element vertices
                
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*dcoeffdx(:,m+1).*dpsidx(:,i).*phi(:,j+4).*invjac_v(:).*jac_m(:);
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*dcoeffdy(:,m+1).*dpsidy(:,i).*phi(:,j+4).*invjac_v(:).*jac_m(:);
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*coeff(:,m+1).*deltapsi(:,i).*phi(:,j+4).*(invjac_v(:).^2).*jac_m(:);
                
            end
        end
    end
    % end of Gauss point loop
end

% 4th sub-element: [-1,0]x[0,1]
subelt=4;
for ivtx = 1:4
    xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
    yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
end
% loop over Gauss points
for igpt = 1:nngpt
    sigpt=s(igpt);
    tigpt=t(igpt);
    sigptloc=(sigpt-1.0e0)/2;
    tigptloc=(tigpt+1.0e0)/2;
    wght=wt(igpt);
    
    % evaluate derivatives etc
    
    [jac_m,~,~,~,~] = deriv(sigpt,tigpt,xl_m,yl_m);
    [~,invjac_v,~,~,~] = deriv(sigptloc,tigptloc,xl_v,yl_v);    
    
    rhs = stoch_gauss_source(sigpt,tigpt,xl_m,yl_m,noarv);
    coeff = stoch_gauss_coeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
    [dcoeffdx,dcoeffdy] = stoch_gauss_gradcoeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
    [phi,~,~] = qderiv(sigpt,tigpt,xl_m,yl_m);
    [~,dpsidx,dpsidy] = qderiv(sigptloc,tigptloc,xl_v,yl_v);
    [dpsi2dx2,dpsi2dy2] = qderiv_2(sigptloc,tigptloc,xl_v,yl_v);
    deltapsi = dpsi2dx2+dpsi2dy2;
    
    for m = 0:noarv
        for j = 1:5  % sub-element vertices
            
            fdem(:,subelt,j,m+1) = fdem(:,subelt,j,m+1) + wght*rhs(:,m+1) .* phi(:,j+4) .* jac_m(:);
            
            for i = 1:9  % element vertices
                
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*dcoeffdx(:,m+1).*dpsidx(:,i).*phi(:,j+4).*invjac_v(:).*jac_m(:);
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*dcoeffdy(:,m+1).*dpsidy(:,i).*phi(:,j+4).*invjac_v(:).*jac_m(:);
                bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+ wght*coeff(:,m+1).*deltapsi(:,i).*phi(:,j+4).*(invjac_v(:).^2).*jac_m(:);
                
            end
        end
    end
    % end of Gauss point loop
end

% manual assembly of subelement contributions
% loop over random variables
for m = 0:noarv
      
    % nodes of error basis with support on macroelement 1
    single_sup1 = [1 8 9];
    % macro ordering of single_sup1
    local_single_sup1 = [1 4 5];
    
    % macroelement 2
    single_sup2 = [2 3 10];
    local_single_sup2 = [1 2 5];
    
    % macroelement 3
    single_sup3 = [4 5 11];
    local_single_sup3 = [2 3 5];
    
    % macroelement 4
    single_sup4 = [6 7 12];
    local_single_sup4 = [3 4 5];
    
    % loop over three error basis functions per macroelement
    for k = 1:3
        % interaction with 9 element solution basis functions
        for kk = 1:9
            bde(:,single_sup1(k),kk,m+1) = bdem(:,1,local_single_sup1(k),kk,m+1);
            bde(:,single_sup2(k),kk,m+1) = bdem(:,2,local_single_sup2(k),kk,m+1);
            bde(:,single_sup3(k),kk,m+1) = bdem(:,3,local_single_sup3(k),kk,m+1);
            bde(:,single_sup4(k),kk,m+1) = bdem(:,4,local_single_sup4(k),kk,m+1);
        end
    end
    
    fde(:,1,m+1)   = fdem(:,1,1,  m+1);
    fde(:,2,m+1)   = fdem(:,2,1,  m+1);
    fde(:,3,m+1)   = fdem(:,2,2,  m+1);
    fde(:,4,m+1)   = fdem(:,3,2,  m+1);
    fde(:,5,m+1)   = fdem(:,3,3,  m+1);
    fde(:,6,m+1)   = fdem(:,4,3,  m+1);
    fde(:,7,m+1)   = fdem(:,4,4,  m+1);
    fde(:,8,m+1)   = fdem(:,1,4,  m+1);
    fde(:,9,m+1)   = fdem(:,1,5,  m+1);
    fde(:,10,m+1)  = fdem(:,2,5,  m+1);
    fde(:,11,m+1)  = fdem(:,3,5,  m+1);
    fde(:,12,m+1)  = fdem(:,4,5,  m+1);
    
end

% compute G-matrices and rhs-multipliers
G = stoch_gmatrices(indset,P,noarv,norv);
[rhs_ind,beta] = stoch_rhs_multipliers(indset,P,norv);

% assemble interior residuals from rhs-contributions
for m=0:noarv
    ind_m = rhs_ind(1,m+1);
    if ind_m>0
        intres(:,(nn*(ind_m-1)+1):(nn*ind_m)) = intres(:,(nn*(ind_m-1)+1):(nn*ind_m)) + ...
            fde(:,:,m+1)*beta(1,m+1);
    end
end

% assemble interior residuals from div(a*grad)-contributions

%   compute local matrices and multiply them by the solution vector
for m = 0:noarv 
    [nzi,nzj,g] = find(G{m+1});
    for t=1:length(nzi)
        i=nzi(t); j=nzj(t);
        for ii = 1:nn
            for jj = 1:9
                intres(:,nn*(i-1)+ii) = ...
                    intres(:,nn*(i-1)+ii) + (g(t)*bde(:,ii,jj,m+1)).*sl_v(:,9*(j-1)+jj);
            end
        end
    end
end


return
